#!/opt/cloudlinux/venv/bin/python3 -bb
# coding=utf-8
#
# Copyright © Cloud Linux GmbH & Cloud Linux Software, Inc 2010-2020 All Rights Reserved
#
# Licensed under CLOUD LINUX LICENSE AGREEMENT
# http://cloudlinux.com/docs/LICENCE.TXT
#

# pylint: disable=no-absolute-import

import getopt
import os
import platform
import shutil
import subprocess
import sys

from sentry import init_wmt_sentry_client
from wmt.common.const import WMT_SCANNER_SERVICE_SOLO, WMT_SCANNER_SERVICE_SOLO_FILE
from clcommon.lib.cledition import is_cl_solo_edition
from wmt.common.utils import setup_logger

logger = setup_logger('daemon_control')

DAEMON_NAME: str = 'cl_wmt_scanner'
_WMT_SERVICE_DIR: str = '/usr/share/web-monitoring-tool/service'
# CL6 service files
_CL6_SERVICE_FILE_PATH: str = '/etc/rc.d/init.d'
_CL6_SERVICE_WMT_FILE: str = 'cl_wmt_scanner'
# CL7/8 service files
_CL7_8_SYSTEMD_UNIT_FILE_PATH: str = '/etc/systemd/system'
_CL7_8_SYSTEMD_WMT_UNIT_FILE: str = 'cl_wmt_scanner.service'

# Utilities
_SERVICE_BIN: str = '/sbin/service'
_CHKCONFIG_BIN: str = '/sbin/chkconfig'
_SYSTEMCTL_BIN: str = '/usr/bin/systemctl'


def usage():
    print('')
    print('Use following syntax to manage LVE Manager install utility:')
    print(sys.argv[0] + " [OPTIONS]")
    print('Options:')
    print(" -i | --install     : install web monitoring tool service")
    print(" -d | --delete      : delete  web monitoring tool service")


def install_wmt_daemon(service_name, is_cl6):
    """
    Install CL wmt daemon
    :return: None
    """
    if is_cl6:
        # CL6
        # /usr/share/cloudlinux/cl_plus/service/cl_wmt_scanner -> /etc/rc.d/init.d/cl_wmt_scanner
        src = os.path.join(_WMT_SERVICE_DIR, service_name)
        dest = os.path.join(_CL6_SERVICE_FILE_PATH, service_name)
        shutil.copy(src, dest)
        os.chmod(dest, 0o755)
    else:
        # CL7, CL8
        # /usr/share/cloudlinux/cl_plus/service/cl_wmt_scanner.service -> /etc/systemd/system/cl_wmt_scanner.service
        src = os.path.join(_WMT_SERVICE_DIR, service_name)
        dest = os.path.join(_CL7_8_SYSTEMD_UNIT_FILE_PATH, service_name)
        shutil.copy(src, dest)
        os.chmod(dest, 0o644)
        subprocess.run('{} daemon-reload'.format(_SYSTEMCTL_BIN), shell=True, executable='/bin/bash')


def enable_wmt_daemon(daemon_name, is_cl6):
    """
    Enable cl_wmt_scanner service
    """
    if is_cl6:
        # CL6
        # Start wmt service
        subprocess.run('{} {} start >/dev/null 2>&1'.format(_SERVICE_BIN, daemon_name),
                       shell=True, executable='/bin/bash')
        subprocess.run('{} --add {} >/dev/null 2>&1'.format(_CHKCONFIG_BIN, daemon_name),
                       shell=True, executable='/bin/bash')
    else:
        # CL7, CL8
        subprocess.run('{} daemon-reload'.format(_SYSTEMCTL_BIN), shell=True, executable='/bin/bash')
        # Start wmt daemon
        subprocess.run('{} enable {}'.format(_SYSTEMCTL_BIN, daemon_name), shell=True, executable='/bin/bash')
        subprocess.run('{} start {}'.format(_SYSTEMCTL_BIN, daemon_name), shell=True, executable='/bin/bash')


def disable_wmt_daemon(daemon_name, is_cl6):
    """
    Disable WMT daemon
    :return:
    """
    if is_cl6:
        # CL6
        subprocess.run('{} {} stop >/dev/null 2>&1'.format(_SERVICE_BIN, daemon_name),
                       shell=True, executable='/bin/bash')
        subprocess.run('{} --del {} >/dev/null 2>&1'.format(_CHKCONFIG_BIN, daemon_name),
                       shell=True, executable='/bin/bash')
    else:
        # CL7, CL8
        subprocess.run('{} kill {} >/dev/null 2>&1'.format(_SYSTEMCTL_BIN, daemon_name),
                       shell=True, executable='/bin/bash')
        subprocess.run('{} disable {} >/dev/null 2>&1'.format(_SYSTEMCTL_BIN, daemon_name),
                       shell=True, executable='/bin/bash')


def stop_remove_wmt_daemon(service_name, is_cl6):
    """
    Remove WMT daemon from system
    :return:
    """
    if is_cl6:
        # CL6
        try:
            # Remove /etc/rc.d/init.d/cl_plus_sender
            src = os.path.join(_CL6_SERVICE_FILE_PATH, service_name)
            os.remove(src)
        except (OSError, IOError):
            pass
    else:
        # CL7, CL8
        try:
            # Remove /etc/systemd/system/cl_plus_sender.service
            src = os.path.join(_CL7_8_SYSTEMD_UNIT_FILE_PATH, service_name)
            os.remove(src)
        except (OSError, IOError):
            pass
        subprocess.run('{} daemon-reload >/dev/null 2>&1'.format(_SYSTEMCTL_BIN), shell=True, executable='/bin/bash')


if __name__ == "__main__":
    init_wmt_sentry_client()
    _is_cl6 = 'el6' in platform.release()
    is_single = is_cl_solo_edition(skip_jwt_check=True)
    if not _is_cl6:
        wmt_service_name = WMT_SCANNER_SERVICE_SOLO_FILE if is_single else _CL7_8_SYSTEMD_WMT_UNIT_FILE
        wmt_daemon_name = WMT_SCANNER_SERVICE_SOLO if is_single else DAEMON_NAME
    else:
        wmt_service_name = _CL6_SERVICE_WMT_FILE
        wmt_daemon_name = DAEMON_NAME
    logger.info(f'Scanner service "{wmt_service_name}"')
    try:
        opts, args = getopt.getopt(sys.argv[1:], "hid", ["help", "install", "delete"])
    except getopt.GetoptError as err:
        # print help information and exit:
        print(str(err))  # will print something like "option -a not recognized"
        usage()
        sys.exit(2)

    try:
        for o, a in opts:
            if o in ("-h", "--help"):
                usage()
                sys.exit()
            elif o in ("-i", "--install"):
                install_wmt_daemon(wmt_service_name, _is_cl6)
                # it`s ok, because this script is used only in .spec
                if is_single:
                    logger.info(f'Solo edition detected, going to enable and start "{wmt_daemon_name}"')
                    enable_wmt_daemon(wmt_daemon_name, _is_cl6)
            elif o in ("-d", "--delete"):
                disable_wmt_daemon(wmt_daemon_name, _is_cl6)
                stop_remove_wmt_daemon(wmt_service_name, _is_cl6)
            else:
                usage()
                sys.exit(2)
    except Exception as e:
        logger.exception(e)
